#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 28-Jan-2004
#################################################################################
#
# Manage Gallery - Script part of Ethereal Realms, designed to manage galleries
# Copyright (C) 2000-2004  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use eRealms::Audit;						# Auditing handler
use eRealms::Database;						# Database handler
use eRealms::Filter;						# Filtering handler
use eRealms::Home;						# File handler
use eRealms::Image;						# Image handling
use eRealms::Login;						# Login functionality
use eRealms::Mail;						# Mail handler
use eRealms::Option;						# Option handlings
use eRealms::Param;						# Parameter control
use eRealms::Table;						# Table control
use eRealms::Template;						# Template handler


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter hash
my $tmpl;							# Template handle

my $expire;							# Expire information
my $upload_css;							# Uploaded CSS
my $upload_hdr;							# Uploaded work header
my $upload_int;							# Uploaded introduction
my $upload_img;							# Uploaded image

my @genres;							# Applicable genres for gallery

my %gal;							# Gallery handler


#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;	
	$database = new eRealms::Database();

	# Connect and fetch
	$database->Connect();


	# Prefetch uploads
	$upload_css = $cgi->upload('UPLOAD_CSS');
	$upload_hdr = $cgi->upload('UPLOAD_HEAD');
	$upload_img = $cgi->upload('UPLOAD_WORK');
	$upload_int = $cgi->upload('UPLOAD_INTRO');

	# Multiple parameters
	@genres = $cgi->param('GGENRES');


	# Parameter handling
	$param    = new eRealms::Param($database, $cgi);
	$param->GetParam();


	# Fetch hash
	$database->GetHashGallery(\%gal);

	# Template handle
	$tmpl = new eRealms::Template(\%gal);


	# Document header
	print $cgi->header();
	$database->DocumentGetContribHeader();


	# Active gallery component
	if ($gal{'SetActive'})
	{
		# Login handle
		$login    = new eRealms::Login($database, $cgi, $param, \%gal);

		# Authenthication
		if ($login->GetVerificationNormal())
		{
			# Retreive expiry
			($expire) = $database->DataGetPuppeteerExpire($param->{'USER'});

			# Verified
			if ($expire eq 'no')
			{
				# Publishing
				if (defined($param->{'PUBLISH'}))
				{
					# Call publishing
					Export($database, $cgi, $param, $tmpl, \%gal);

					# Cleanup
					$param->Cleanup('PUBLISH');
				}
				
				# Work specific
				if (defined($param->{'WORK'}))	
				{
					# Editing
					if ((defined($param->{$gal{'TxtDetails'}}))
					 || (defined($param->{'EDIT_WORK'})))
					{
						# Detailed view
						WorkDetail($database, $cgi, $param, $tmpl, \%gal);
					}

					# Creation and management
					else
					{
						# Default behaviour
						WorkSummary($database, $cgi, $param, $tmpl, \%gal, $upload_img);
					}
				}

				# News entries and updates
				elsif (defined($param->{'NEWS'}))
				{
					# News creation
					NewsEntry($database, $cgi, $param, $tmpl, \%gal);
				}

				# News entries and updates
				elsif (defined($param->{'LINK'}))
				{
					# News creation
					LinkEntry($database, $cgi, $param, $tmpl, \%gal);
				}


				# Gallery specific
				else
				{
					# Editing
					if ((defined($param->{'GALLERY'}))
					 && ((defined($param->{$gal{'TxtDetails'}}))
					 ||  (defined($param->{'EDIT_GALLERY'}))))
					{
						# Detailed view
						GalleryDetail($database, $cgi, $param, $tmpl, \%gal, \@genres, $upload_css, $upload_hdr, $upload_int);
					}

					# Creation and management
					else
					{
						# Default behaviour
						GallerySummary($database, $cgi, $param, $tmpl, \%gal);
					}
				}
			}

			else
			{
				# Non verified
				$tmpl->Show('TmplMsg', MMESSAGE=>$gal{'MsgExpire'});
			}
		}
	}

	else
	{
		# Non-active
		$tmpl->Show('TmplMsg', MMESSAGE=>$gal{'MsgDisabled'});
	}

	# Document footer
	$database->DocumentGetContribFooter();


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Check Data
#
# This subroutine exists to verify information that is sent to it.   Basically
# this makes a more consistent interface and simplifies matters.

sub CheckData
{
	#####################
	# Data members
	my $choice   = shift;					# Choice of the matter
	my $value    = shift;					# Value to verify
	my $limit    = shift;					# Optional minimum limit


	#####################
	# Program area

	# Text field check
	if ($choice == 0)
	{
		# Allows A-Z and spaces
		return 1 if ($value =~ /^\w[\w\s\'\"\.\,\:\;\?\-]{$limit,}$/);
	}

	# Email check
	elsif ($choice == 1)
	{
		# Has the semblence of an Email address
		return 1 if ($value =~ /^\w(\w|\.|\-)*?\@(\w|\-)*?\..*?/s);
	}


	# If all else fail
	return 0;
}

# Alias for E-Eail validity
sub CheckEMail
{ CheckData(1, shift); }

# Alias for Lenght
sub CheckString
{ CheckData(0, shift, shift); }


#####################
# Export
#
# Exports all text works into one singular document that can be used to produce
# one PDF file in novel like format.

sub Export
{
	#####################
	# Data members

	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $gal      = shift;					# Gallery hash

	my $option;						# Option handler
	my $page;						# Page handle
	my $res;						# Results set
	my $statement;						# Query handle
	
	my $chapter;						# Applied template
	my $cover;						# Cover page
	my $dir;						# Directory
	my $file;						# Singular file
	my $files = '';						# File for import

	my $fsort;						# Sorting application
	my $gsort;						# Gallery sorting
	my $gtype;						# Gallery Type

	my @files;						# Array of files
	
	my %system;						# System handle


	#####################
	# Program area

	# Associate with hash
	$database->GetHashSystem(\%system);

	# Directory
	$dir = $gal->{'SetRoot'} . '/' . $param->{'GID'};

	# Create instance
	$option = new eRealms::Option();
	$page   = new eRealms::Home($database, $dir);


	# Fetch sort information
	($gtype, $gsort) = $database->DataGetGalleryType($param->{'GID'}, $param->{'USER'});

	# Determine sort order
	$fsort     = $option->GallerySortReal($gsort);	

	# Fetch and export work
	($cover)   = $database->DocumentGetGalleryCover();
	($chapter) = $database->DocumentGetGalleryChapter();


	# Works
	$database->Pull(\$statement, "SELECT 
		  g.GalleryPenName AS \"GalleryPenName\", 
		  g.GalleryName    AS \"GalleryName\", 
		  g.GalleryID      AS \"GalleryID\", 
		  w.WorkText       AS \"WorkText\", 
		  w.WorkID         AS \"WorkID\", 
		  w.WorkName       AS \"WorkName\",
		  getDate(w.WorkTimestamp) AS \"WorkTimestamp\"
		FROM ethereal.Gallery g, ethereal.Work w
		WHERE g.GalleryID=w.GalleryID
	        AND g.GalleryType='text'
		AND w.WorkText IS NOT NULL
		AND g.GalleryID=?
		ORDER BY w.$fsort",
	
	  $param->{'GID'}
	);


	# Sorting
	if ($res = $statement->fetchrow_hashref())
	{
		# Cover page
		$cover =~ s/MAUTHOR/$res->{'GalleryPenName'}/gs;
		$cover =~ s/MGID/$res->{'GalleryID'}/gs;
		$cover =~ s/MGNAME/$res->{'GalleryName'}/gs;
		$cover =~ s/MTITLE/$res->{'WorkName'}/gs;
		$cover =~ s/MWID/$res->{'WorkID'}/gs;


		# Publish
		$page->FileWrite('cover.html', $cover);

		# Push filename
		push(@files, 'cover.html');


		# Loop and publish
		# Continue as long as there are works
		do
		{
			# Publishing handler
			push(@files, Publish($page, $res, $chapter, \%system, $gal));

		} while ($res = $statement->fetchrow_hashref());
	}

	# End
	$statement->finish();


	# Create filenames
	# Cycle and populate
	foreach $file (@files)
	{
		# Annotate
		$files = $files . "$dir/$file ";
	}


	# Generate files
	system("$system{SetExport} $system{SetExportNovel} $dir/novel.pdf $files");


	# Unlink all files
	foreach $file (@files)
	{ 
		# Cleanup
		$page->FileUnlink($file);
	}
}

#####################
# Publish
#
# An exact copy of the code that is used in the gallery.pl file in order to
# publish works.   This one is self-contained as to prevent any chance of tainting
# information.

sub Publish
{
	#####################
	# Data members

	my $page     = shift;					# Page handler
	my $results  = shift;					# Results 
	my $export   = shift; 					# Changeable template
	my $system   = shift; 					# System hash
	my $gal      = shift; 					# Gallery hash

	my $file;						# Filename
	my $data;						# Data file

	
	#####################
	# Program area

	# Set filename
	$file = 'exp' . $results->{'WorkID'} . '.html';

	# Add in spaces
	$data =  $results->{'WorkText'};
	$data =~ s/\n/<BR>/gs;
	

	# Search and replace
	$export =~ s/MAUTHOR/$results->{'GalleryPenname'}/gs;
	$export =~ s/MDATE/$results->{'WorkTimestamp'}/gs;
	$export =~ s/MTITLE/$results->{'WorkName'}/gs;
	$export =~ s/MBODY/$data/gs;
						
	# Write temporary file
	$page->FileWrite($file, $export);


	# Return value
	return $file;
}


#####################
# Gallery Detail
#
# Gallery detail, will handle the finer points of a gallery itself.  This
# includes setting names, genres et cetera.

sub GalleryDetail
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $gal      = shift;					# Gallery hash
	my $genres   = shift;					# Array of genres
	my $css_over = shift;					# CSS override
	my $hdr_over = shift;					# Work header override
	my $int_over = shift;					# Intro override

	my $filter;						# Filter handler
	my $option;						# Option handler
	my $page;						# File writing library
	my $statement;						# Database query statement
	my $table;						# Table creation

	my $contents;						# Contents of file
	my $mime;						# Mime types
	
	my $descr;						# Description of types
	my $exist;						# Existence of value
	my $id;							# Gallery number
	my $name;						# Gallery name
	my $res;						# Results
	my $time;						# Current time

	my $url;						# Self referencing url	
	my $idele;						# Deleting compilation
	my $ilink;						# Link handling
	my $imain;						# Main inline parameters
	my $inews;						# News handling
	my $ipubl;						# Publishing
	my $iwork;						# Work inline parameters

	my $comment;						# Comment options
	my $display;						# Display options
	my $genre;						# Genre options
	my $privacy;						# Gallery privacy
	my $puppet;						# Puppet pen names
	my $rating;						# Rating options
	my $sort;						# Sorting options
	my $submt;						# Submit button
	my $type;						# Type options

	my $count = 0;						# Generic counter

	my @comment;						# Comment options array
	my @display;						# Display options array
	my @estab;						# Established genres
	my @genre;						# Genre options array
	my @privacy;						# Privacy options array
	my @puppet;						# Puppet list 
	my @rating;						# Rating options array
	my @sort;						# Sorting options array
	my @type;						# Type options array


	#####################
	# Program area	

	# Initiate instances
	$option = new eRealms::Option(); 
	$table  = new eRealms::Table($cgi);


	# Retreive ID
	($id, $name) = split(/ - /, $param->{'GALLERY'});


	# File uploads
	# CSS Override
	if (defined($css_over))
	{
		# Create instance
		$page  = new eRealms::Home($database, "$gal->{SetRoot}/$id");

		# Determin MIME type
		$mime = $cgi->uploadInfo($param->{'UPLOAD_CSS'})->{'Content-Type'};

		# Valid MIME type
		if ($mime =~ /^text/i)
		{
			# Initial set
			$contents = '';
			
			# Retreive image
			while (<$css_over>) { $contents = "$contents$_"; }

			# File writing
			# Write image
			$page->FileWrite("gal.css", $contents);
		}
	}

	# Work header override
	if (defined($hdr_over))
	{
		# Determin MIME type
		$mime = $cgi->uploadInfo($param->{'UPLOAD_HEAD'})->{'Content-Type'};

		# Valid MIME type
		if ($mime =~ /^text/i)
		{
			# Initial set
			$contents = '';
			
			# Retreive image
			while (<$hdr_over>) { $contents = "$contents$_"; }


			# Safety check
			$contents = (length($contents) > 15) ? $contents : undef;

			# Update database entry
			$database->Quick("UPDATE Gallery
				SET GalleryWrkHeader=?
				WHERE GalleryID=?",

			 $contents,
		 	 $id
			);
		}
	}

	# Introduction override
	if (defined($int_over))
	{
		# Determin MIME type
		$mime = $cgi->uploadInfo($param->{'UPLOAD_INTRO'})->{'Content-Type'};

		# Valid MIME type
		if ($mime =~ /^text/i)
		{
			# Initial set
			$contents = '';
			
			# Retreive image
			while (<$int_over>) { $contents = "$contents$_"; }


			# Safety check
			$contents = (length($contents) > 15) ? $contents : undef;

			# Update database entry
			$database->Quick("UPDATE Gallery
				SET GalleryIntro=?
				WHERE GalleryID=?",

			 $contents,
		 	 $id
			);
		}
	}

	
	# Compilation
	# Remove compilation
	if (defined($param->{'COMP_DELETE'}))
	{
		# Create instance
		$page  = new eRealms::Home($database, "$gal->{SetRoot}/$id");

		# Remove file
		$page->FileUnlink('novel.pdf');
	}

	
	# Information modification
	# Creation of news items
	if ((defined($param->{'SAVE_NEWS'}))
	 && (length($param->{'SAVE_NEWS'}) > 15))
	{
		# Create instance
		$filter = new eRealms::Filter();

		# Filter text
		$param->{'SAVE_NEWS'} = $filter->SafeHTML($param->{'SAVE_NEWS'});
		$param->{'SAVE_NEWS'} = substr(DBI::neat($param->{'SAVE_NEWS'}, 452), 1, -1);

		# Simplify HTML
		$param->{'SAVE_NEWS'} =~ s/\n/<BR>/gs;

		
		# Create entry
		$database->Quick("INSERT INTO Comment
			(GalleryID,
			 WorkID,
			 PuppetName,
			 PuppeteerLogin,
			 CommentPost,
			 CommentRating)
			VALUES(?,1,getPuppet(?),?,?,0)",

		 $id,
		 $param->{'USER'},
		 $param->{'USER'},
		 $param->{'SAVE_NEWS'}
		);

		# Safety cleanup
		$param->Cleanup('EDIT_GALLERY');
	}
	
	# Modify existing information
	if ((defined($param->{'EDIT_GALLERY'}))
	 && ($param->{'EDIT_GALLERY'} eq 'True'))
	{
		# Initial type setup
		if (defined($param->{'GTYPE'}))
		{
			# Determine type
			$type = $option->GalleryType($gal->{'OptType'}, $param->{'GTYPE'});

			# Evaluate block
			eval
			{
				# Update database
				$database->Write("UPDATE Gallery
					SET GalleryType=?,
					    GalleryUpdated=LOCALTIMESTAMP
					WHERE GalleryID=?
					  AND PuppeteerLogin=?",

				 $type,
				 $id,
				 $param->{'USER'}
				);

				# Add in necessary genre
				$database->Write("INSERT INTO GalleryGenre
					(GalleryID,
					 GenreName,
					 GenreOrder)
					VALUES(?,?,?)",

				 $id,
				 $gal->{'SetGenre'},
				 $type
				);

				# Commit changes
				$database->Commit();
			};

			# If block failed
			if ($@)
			{
				# Issue warning
				warn("Transaction aborted: $@");

				# Undo incomplete changes
				$database->Rollback();
			}

			# Cleanup
			$param->Cleanup('GTYPE');
		}

		# Standard
		else
		{
			# Initiate handle
			$filter = new eRealms::Filter();

			# Existence
			($exist) = $database->DataGetGalleryID($param->{'GNAME'});


			# Main update
			# Description
			$param->{'GDESCR'} = $filter->StripHTML($param->{'GDESCR'});
			$param->{'GDESCR'} = substr(DBI::neat($param->{'GDESCR'}, 452), 1, -1);
			$param->{'GDESCR'} = (length($param->{'GDESCR'}) > 15) ? $param->{'GDESCR'} : undef;

			# Determin value
			$name    = (defined($exist)) ? $name : $param->{'GNAME'};

			# Options
			$comment = $option->GalleryComment($gal->{'OptComment'}, $param->{'GCOMMENT'});
			$privacy = $option->GalleryPrivacy($gal->{'OptPrivacy'}, $param->{'GPRIVACY'});
			$sort    = $option->GallerySort($gal->{'OptSort'}, $param->{'GSORT'});


			# Update
			eval
			{
				# Entry
				$database->Write("UPDATE Gallery
					SET GalleryName=?,
					    GalleryRating=?,
					    GalleryDescription=?,
					    GalleryPenName=?,
					    GalleryDisplay=?,
			 		    GalleryComments=?,
			 		    GalleryPrivacy=?,
					    GallerySort=?,
					    GalleryUpdated=LOCALTIMESTAMP
					WHERE GalleryID=?
					  AND PuppeteerLogin=?",

				 $name,
				 $param->{'GRATING'},
				 $param->{'GDESCR'},
				 $param->{'GPUPPET'},
				 $param->{'GDISPLAY'},
				 $comment,
				 $privacy,
				 $sort,
				 $id,
				 $param->{'USER'}
				);

				# Update master work
				$database->Write("UPDATE Work
					SET WorkName=?
					WHERE WorkID < 2
					  AND GalleryID=?",

				 $name,
				 $id
				);


				# Genre handling
				$database->Write("DELETE
					FROM GalleryGenre
					WHERE GalleryID=?", $id);


				# Loop and re-add
				foreach $genre (@{$genres})
				{
					# Exit loop if necessary
					last if ($count > 2);

					# Insert individual choices
					$database->Write("INSERT INTO GalleryGenre
						(GalleryID,
						 GenreName,
						 GenreOrder)
						VALUES(?,?,?)",

					 $id,
					 $genre,
					 $param->{'GORDER'}
					);

					# Increment
					$count++;
				}


				# Commit transaction
				$database->Commit();
			};

			# If block failed
			if ($@)
			{
				# Issue warning
				warn("Transaction aborted: $@");

				# Undo incomplete changes
				$database->Rollback();
			}

			# Parameter
			$param->{'GALLERY'} = $id . ' - ' . $name;
		}
	}

	# Cleanup
	$param->Cleanup($gal->{'TxtDetails'}, 'GORDER');



	# Information gathering
	# Prepare main body
	$database->Pull(\$statement, "SELECT
		 GalleryPenName     AS \"GalleryPenName\",
		 GalleryName        AS \"GalleryName\",
		 GalleryRating      AS \"GalleryRating\",
		 GalleryDescription AS \"GalleryDescription\",
		 GalleryComments    AS \"GalleryComments\",
		 GallerySort        AS \"GallerySort\",
		 GalleryType        AS \"GalleryType\",
		 GalleryPrivacy     AS \"GalleryPrivacy\",
		 GalleryDisplay     AS \"GalleryDisplay\",
		 getDate(GalleryCreated) AS \"GalleryCreated\",
		 getDate(GalleryUpdated) AS \"GalleryUpdated\"
		FROM Gallery
		WHERE GalleryID=?
		  AND PuppeteerLogin=?",

	 $id,
	 $param->{'USER'}
	);


	# Information display
	# Fetch results
	if ($res = $statement->fetchrow_hashref())
	{
		# Start Form
		# Regular
		unless (($res->{'GalleryType'} eq 'comic')
		 || ($res->{'GalleryType'} eq 'journal'))
	 	{
			# Initial, image and text galleries
			print $cgi->start_form();
			print $param->EmbedNormal($param->Flat(),
				GORDER       => $res->{'GalleryType'},
				EDIT_GALLERY => 'True'
			 );
		}

		# Pre-populate certain choices
		else
		{
			# Live journals and webcomics
			print $cgi->start_multipart_form();
			print $param->EmbedNormal($param->Flat(),
				GORDER       => $res->{'GalleryType'},
				GSORT        => $option->GallerySortRev($gal->{'OptSort'}, 'date'),
				GDISPLAY     => $res->{'GalleryDisplay'},
				EDIT_GALLERY => 'True'
			 );
		}

		

		
		# Verify initial access
		if ($res->{'GalleryType'} eq 'initial')
		{
			# Pull document
			($descr) = $database->DocumentGetGalleryType();

			# Pull options
			$option->Split(\@type, $gal->{'OptType'});


			# Generate widgets
			$submt = $cgi->submit($gal->{'TxtSelect'});
			$type  = $cgi->popup_menu('GTYPE', \@type, $type[0]);
			
			# Replace
			$descr =~ s/WGSUBMT/$submt/gs;
			$descr =~ s/WGTYPES/$type/gs;


			# Display document
			print $descr, "\n";
		}


		# Default behaviour
		else
		{
			# Back track
			$url   = $cgi->url(-relative=>1);
			$imain = $param->EmbedInline(
				USER    => $param->{'USER'}, 
				CRYPT   => $param->{'CRYPT'}
		 	 );
			$idele = $param->EmbedInline(
				USER    => $param->{'USER'}, 
				CRYPT   => $param->{'CRYPT'},
				GALLERY => $param->{'GALLERY'},
				GID     => $id,
			 	COMP_DELETE  => 'True',
				EDIT_GALLERY => 'False'	
			);
			$ilink = $param->EmbedInline(
				USER    => $param->{'USER'}, 
				CRYPT   => $param->{'CRYPT'},
				GALLERY => $param->{'GALLERY'},
				GID     => $id,
			 	LINK    => 'True'	
			);
			$inews = $param->EmbedInline(
				USER    => $param->{'USER'}, 
				CRYPT   => $param->{'CRYPT'},
				GALLERY => $param->{'GALLERY'},
				GID     => $id,
			 	NEWS    => 'True'	
			);
			$ipubl = $param->EmbedInline(
				USER    => $param->{'USER'}, 
				CRYPT   => $param->{'CRYPT'},
				GALLERY => $param->{'GALLERY'},
				GID     => $id,
			 	PUBLISH => 'True',
				EDIT_GALLERY => 'False'	
			);
			$iwork = $param->EmbedInline(
				USER    => $param->{'USER'}, 
				CRYPT   => $param->{'CRYPT'},
				GID     => $id,
			 	WORK    => 'True'	
			);

			# Genres
			# Established
			$database->GetListEstablished(\@estab, $id);
	
			# Total
			$database->GetListGenre(\@genre, $res->{'GalleryType'});	


			# Gather information
			$option->Split(\@comment, $gal->{'OptComment'});
			$option->Split(\@display, $gal->{'OptDisplay'});
			$option->Split(\@privacy, $gal->{'OptPrivacy'});
			$option->Split(\@rating,  $gal->{'OptRating'});
			$option->Split(\@sort,    $gal->{'OptSort'});

			# Puppet
			$database->GetListPuppetYours(\@puppet, $param->{'USER'});


			# Initial discovery
			$comment = $option->GalleryCommentRev($gal->{'OptComment'}, $res->{'GalleryComments'});
			$privacy = $option->GalleryPrivacyRev($gal->{'OptPrivacy'}, $res->{'GalleryPrivacy'});
			$sort    = $option->GallerySortRev($gal->{'OptSort'}, $res->{'GallerySort'});

			# Generate widgets
			$puppet  = $cgi->popup_menu('GPUPPET',   \@puppet,  $res->{'GalleryPenName'});
			$rating  = $cgi->popup_menu('GRATING',   \@rating,  $res->{'GalleryRating'});

			# Checkbox group
			$genre   = $cgi->checkbox_group(
			  -name      => 'GGENRES',
		    	  -values    => \@genre,
			  -defaults  => \@estab,
		  	  -linebreak => 'true');

			$comment = $cgi->radio_group('GCOMMENT', \@comment, $comment, 'true');
			$display = $cgi->radio_group('GDISPLAY', \@display, $res->{'GalleryDisplay'}, 'true');
			$privacy = $cgi->radio_group('GPRIVACY', \@privacy, $privacy, 'true'); 
			$sort    = $cgi->radio_group('GSORT',    \@sort,    $sort, 'true');


			# Display information
			# Title
			$tmpl->ShowMain($gal->{'TagGalleryDetail'});

			# Link
			print "<CENTER>";
			print "<A HREF=\"$url$imain\">$gal->{'TagGalListing'}</A> ",
			      "<A HREF=\"$url$iwork\">$gal->{'TagGalWork'}</A>&nbsp;&nbsp;&nbsp; ",
			      "<A HREF=\"$url$ilink\">$gal->{'TagGalLinks'}</A> ",
			      "<A HREF=\"$url$inews\">$gal->{'TagGalNews'}</A>";
			print "&nbsp;&nbsp;&nbsp;<A HREF=\"$url$ipubl\">$gal->{'TagGalPublish'}</A> " if ($res->{'GalleryType'} eq 'text');
			print "<A HREF=\"$url$idele\">$gal->{'TagGalCDel'}</A>" if (-f "$gal->{SetRoot}/$id/novel.pdf");
			print "</CENTER><BR>";


			# Table top
			$table->MakeTop();

			# Key identifier	
			$table->MakeSingle($tmpl->PassTitle($res->{'GalleryName'}));	
			$table->MakeBlank();


			# Basic identifiers
			$table->MakeValid(
				$tmpl->PassSub($gal->{'TagGalName'}),
				$cgi->textfield('GNAME', $res->{'GalleryName'}, 25, 30)
			 );
			$table->MakeValid($tmpl->PassSub($gal->{'TagGalRating'}), $rating);
			$table->MakeValid($tmpl->PassSub($gal->{'TagGalGenre'}), $genre);
			$table->MakeBlank();


			# Alternate identity
			$table->MakeValid($tmpl->PassSub($gal->{'TagGalPenName'}), $puppet);
			$table->MakeBlank();
			

			# Settings
			$table->MakeValid($tmpl->PassSub($gal->{'TagGalComment'}), $comment);
			$table->MakeBlank();
			$table->MakeValid($tmpl->PassSub($gal->{'TagGalPrivacy'}), $privacy);
			$table->MakeBlank();

			# Image and Text specific
			if (($res->{'GalleryType'} eq 'comic')
			 || ($res->{'GalleryType'} eq 'journal'))
	 		{
				# Upload fields
				$table->MakeSingle($tmpl->PassTitle($gal->{'TagGalOverride'}));
				$table->MakeValid(
					$tmpl->PassSub($gal->{'TagAddCSSOver'}),
					$cgi->filefield('UPLOAD_CSS', '', 25, 140)
				 );
				$table->MakeValid(
					$tmpl->PassSub($gal->{'TagAddIntroOver'}),
					$cgi->filefield('UPLOAD_INTRO', '', 25, 140)
				 );
				$table->MakeValid(
					$tmpl->PassSub($gal->{'TagAddWorkOver'}),
					$cgi->filefield('UPLOAD_HEAD', '', 25, 140)
				 );
				$table->MakeBlank();
			}

			# Allow for more malleability
			else
			{
				# More options
				$table->MakeValid($tmpl->PassSub($gal->{'TagGalSort'}),    $sort);
				$table->MakeBlank();
				$table->MakeValid($tmpl->PassSub($gal->{'TagGalDisplay'}), $display);
				$table->MakeBlank();
			}

			# Description
			$table->MakeSingle($tmpl->PassTitle($gal->{'TagGalDescription'}));
			$table->MakeSingle($cgi->textarea(
					-name    => 'GDESCR',
					-rows    => 3,
					-columns => 75,
					-wrap    => 'SOFT',
					-default => $res->{'GalleryDescription'})
				);
			$table->MakeBlank();


			# Submission widgets
			$table->MakeValid($cgi->submit($gal->{'TxtUpdate'}), $cgi->reset($gal->{'TxtReset'}));

			# Table bottom
			$table->MakeBlank();
			$table->MakeSingle($tmpl->Pass('TmplStatus',
				MGALLERY => $id,
				MGALTYPE => ucfirst($res->{'GalleryType'}),
				MCREATED => $res->{'GalleryCreated'},
				MUPDATED => $res->{'GalleryUpdated'},
				MGALLINK => "$gal->{LnkGallery}/view/$id$imain")
			 );
			$table->MakeBottom();
		}

		# End of form
		print $cgi->end_form(), "\n";
	}

	# Finish query
	$statement->finish();
}


#####################
# Gallery Summary
#
# Gallery summary is used to display a list of all galleries for the
# user and allow them to modify them as need be. 

sub GallerySummary
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $gal      = shift;					# Gallery hash

	my $audit;						# Auditing handle
	my $filter;						# Filter handle
	my $page;						# File handler
	my $table;						# Table creation

	my $buttons;						# Management buttons
	my $listing;						# List of galleries

	my $exist;						# Existence
	my $id;							# Last insert ID
	my $message;						# Notice message
	my $puppet;						# Default puppet name

	my @galleries;						# Array of galleries


	#####################
	# Program area	

	# Create instance
	$audit = new eRealms::Audit($database, $cgi, $param);

	# Information handling
	# Additions with names greater then 3 
	if ((defined($param->{$gal->{'TxtCreate'}}))
	 && (CheckString($param->{'ADD_GALLERY'}, 2)))
	{
		# Check for existence
		($exist)  = $database->DataGetGalleryID($param->{'ADD_GALLERY'});

		# Name not already taken
		unless (defined($exist))
		{
			# Evaluate block
			eval
			{
				# Retreive base information
				($puppet) = $database->DataGetDefault($param->{'USER'});


				# Creation
				$database->Write("INSERT INTO Gallery
					(PuppeteerLogin,
					 GalleryName,
					 GalleryRating,
					 GalleryDisplay,
					 GalleryPenName)
					VALUES(?,?,?,?,?)",

				 $param->{'USER'},
				 $param->{'ADD_GALLERY'},
				 $gal->{'SetRating'},
				 $gal->{'SetDisplay'},
				 $puppet
				);
	

				# Determine ID
				($id) = $database->DataGet("SELECT CURRVAL('seqGallery')");

				# Create system level works
				# Gallery Comments
				$database->Write("INSERT INTO Work
					(GalleryID,
					 WorkID,
					 WorkName)
					VALUES(?,0,?)",

				 $id,
				 $param->{'ADD_GALLERY'}
				);

				# Artists news
				$database->Write("INSERT INTO Work
					(GalleryID,
					 WorkID,
					 WorkName)
					VALUES(?,1,?)",

				 $id,
				 $param->{'ADD_GALLERY'}
				);


				# Remove if exists
				# Directory
				if (-d "$gal->{SetRoot}/$id")
				{
					# Create instance
					$page = new eRealms::Home($database);

					# Remove directory
					$page->DirClear("$gal->{SetRoot}/$id");
				}

				# File
				if (-f "$gal->{SetRoot}/$id")
				{
					# Unlink
					unlink("$gal->{SetRoot}/$id");
				}


				# Attempt to create
				unless (mkdir("$gal->{SetRoot}/$id", 0775))
				{
					# Rollback
					$database->Rollback();

					# Signal notice
					$message = $gal->{'MsgError'} . $!;
				}

				else
				{
					# Auditing
					$audit->GalleryAdd($id);

					# Message of success
					$message = $gal->{'MsgCreated'};
				}

				# Commit
				$database->Commit();
			};

			# If block failed
			if ($@)
			{
				# Issue warning
				warn("Transaction aborted: $@");

				# Undo incomplete changes
				$database->Rollback;
			}
		}

		# Notify
		else
		{
			# Existence message
			$message = $gal->{'MsgExists'};
		}
	}

	# Removal
	if ((defined($param->{$gal->{'TxtDelete'}}))
	 && (defined($param->{'CONFIRM'})))
	{
		# Retreive ID
		($id)    = split(/ - /, $param->{'GALLERY'});

		# Confirm
		($exist) = $database->DataGetGalleryName($id, $param->{'USER'});


		# Systematically remove
		if ((defined($id))
		 && (defined($exist)))
		{
			# Evaluate block
			eval
			{
				# Create instance
				$page = new eRealms::Home($database);

				# Remove directory
				$page->DirClear("$gal->{SetRoot}/$id");


				# Remove database entry
				$database->Write("DELETE FROM Gallery
					WHERE GalleryID=?
					  AND PuppeteerLogin=?",
				 $id,
				 $param->{'USER'}
				);

				# Signal notice
				$message = $gal->{'MsgRemoved'};


				# Record change
				$audit->GalleryRem($id);
			};

			# If block failed
			if ($@)
			{
				# Issue warning
				warn("Transaction aborted: $@");

				# Undo incomplete changes
				$database->Rollback;
			}
		}
	}

	# Always cleanup
	$param->Cleanup('CONFIRM');


	# Information display
	# Initiate handle
	$table = new eRealms::Table($cgi);


	# Retrieve list
	$database->GetList(\@galleries, "SELECT GalleryId || ' - ' || GalleryName AS \"Gallery\" 
		 FROM Gallery 
		 WHERE PuppeteerLogin=?
		 ORDER BY GalleryName", $param->{'USER'});

	# Widget generation
	$listing = $cgi->scrolling_list('GALLERY', \@galleries, $galleries[0], 5);
	$buttons = $cgi->submit($gal->{'TxtDetails'}) . '<BR><BR>'
		.  $cgi->submit($gal->{'TxtDelete'})
		.  $cgi->checkbox(-name=>'CONFIRM', -label=>'');


	# Display
	# Title
	$tmpl->ShowMain($gal->{'TagGallery'});

	# Form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat()), "\n";

	# Table top
	$table->MakeTop();

	# Notice messages
	if (defined($message))
	{
		# Display through template
		$table->MakeSingle($tmpl->Pass('TmplNotice', MNOTICE=>$message));
		$table->MakeBlank();
	}


	# Gallery listing
	$table->MakeSingle($tmpl->PassSub($gal->{'TagSelectGallery'}));
	$table->MakeValid($listing, $buttons, 'middle');
	$table->MakeBlank();

	# Gallery creation
	$table->MakeSingle($tmpl->PassTitle($gal->{'TagAddGallery'}));
	$table->MakeValid(
		$tmpl->PassSub($gal->{'TagAddGalleryName'}),
		$cgi->textfield('ADD_GALLERY', '', 25, 30)
	 );
	$table->MakeSingle($cgi->submit($gal->{'TxtCreate'}));
	
	# Table Bottom
	$table->MakeBottom();

	# End form
	print $cgi->end_form(), "\n";
}


#####################
# Quick Links Edit
#
# Link management allows for the additional sidebar entry that is used
# to show off certain links of interest to the author of a specific
# gallery.

sub LinkEntry
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $gal      = shift;					# Gallery hash

	my $res;						# Results
	my $statement;						# SQL statement

	my $table;						# Table creation

	my $url;						# Self referencing link
	my $iback;						# Inline main menu


	#####################
	# Program area	

	# Data management
	# Addition
	if (defined($param->{$gal->{'TxtCreate'}}))
	{
		# Basic safety
		if ((defined($param->{'LNKNAME'}))
		 && (defined($param->{'LNKADD'}))
		 && (length($param->{'LNKNAME'}) > 3)
		 && ($param->{'LNKADD'} =~ /^http:\/\/.+/))
	 	{
			# Insert new link
			$database->Quick("INSERT INTO Link
				(PuppeteerLogin,
				 GalleryID,
				 LinkName,
				 LinkAddress)
				VALUES(?,?,?,?)",

			 $param->{'USER'},
			 $param->{'GID'},
			 $param->{'LNKNAME'},
			 $param->{'LNKADD'}
			);
		}
	}

	
	# Create instances
	$table  = new eRealms::Table($cgi);

	# Back track
	$url   = $cgi->url(-relative=>1);
	$iback = $param->EmbedInline(
		USER  => $param->{'USER'}, 
		CRYPT => $param->{'CRYPT'},
	 	GID   => $param->{'GID'}
 	 );


	# Display
	# Title
	$tmpl->ShowMain($gal->{'TagLinks'});

	# Link
	print "<CENTER><A HREF=\"$url$iback\">$gal->{'TagBack'}</A></CENTER><BR>";


	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat()), "\n";


	# Table top
	$table->MakeTop();

	# Gallery listing
	$table->MakeSingle($tmpl->PassTitle($gal->{'TagLinksCurrent'}));

	# Prepare and execute
	$database->Pull(\$statement, "SELECT
		 LinkID      AS \"LinkID\", 
		 LinkName    AS \"LinkName\", 
		 LinkAddress AS \"LinkAddress\"
		FROM Link
		WHERE PuppeteerLogin=?
		  AND GalleryID=?
		ORDER BY LinkName",

	 $param->{'USER'},
	 $param->{'GID'}
 	);

	# Loop and display current links
	while ($res = $statement->fetchrow_hashref())
	{
		# Display singular row
		unless (defined($param->{"LNK_$res->{LinkID}"}))
		{
			# Simple entry/checkbox style
			$table->MakeValid(
				$tmpl->Pass('TagLinksSingle', MNAME=>$res->{'LinkName'}, MLINK=>{'LinkAddress'}),
				$cgi->checkbox(-name=>"LNK_$res->{LinkID}", -label=>'')
			 );
		}

		# User requested removal
		else
		{
			# Remove entry
			$database->Quick("DELETE FROM Link
				WHERE PuppeteerLogin=?
				  AND LinkID=?",

			 $param->{'USER'},
			 $res->{'LinkID'}
			);
		}
	}

	# End query
	$statement->finish();

	# Spacer
	$table->MakeBlank();
	$table->MakeValid('&nbsp;', $cgi->submit($gal->{'TxtDelete'}));

	# Additions
	$table->MakeSingle($tmpl->PassTitle($gal->{'TagLinksAddition'}));

	# Name and address fields
	$table->MakeValid(
		$tmpl->PassSub($gal->{'TagLinksName'}),
		$cgi->textfield('LNKNAME', '', 20, 20)
	 );
	$table->MakeValid(
		$tmpl->PassSub($gal->{'TagLinksAddress'}),
		$cgi->textfield('LNKADD', 'http://', 40, 240)
	 );

	
	# Submit and closure
	$table->MakeValid('&nbsp;', $cgi->submit($gal->{'TxtCreate'}));
	$table->MakeBottom();

	# End form
	print $cgi->end_form(), "\n";
}


#####################
# News Entry
#
# Allows for the addition of news entries into the galleries front page
# as this is only a variant of the comments section, formatting will be
# more limited.

sub NewsEntry
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $gal      = shift;					# Gallery hash

	my $table;						# Table creation

	my $url;						# Self referencing link
	my $iback;						# Inline main menu


	#####################
	# Program area	

	# Create instances
	$table  = new eRealms::Table($cgi);

	# Always cleanup
	$param->Cleanup('NEWS');

	# Back track
	$url   = $cgi->url(-relative=>1);
	$iback = $param->EmbedInline(
		USER  => $param->{'USER'}, 
		CRYPT => $param->{'CRYPT'},
	 	GID   => $param->{'GID'}
 	 );


	# Display
	# Title
	$tmpl->ShowMain($gal->{'TagNews'});

	# Link
	print "<CENTER><A HREF=\"$url$iback\">$gal->{'TagBack'}</A></CENTER><BR>";


	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat(), EDIT_GALLERY=>'True'), "\n";


	# Table top
	$table->MakeTop();

	# Gallery listing
	$table->MakeSingle($tmpl->PassTitle($gal->{'TagNewsText'}));
	$table->MakeValid($cgi->textarea(
		-name    => 'SAVE_NEWS',
		-rows    => 15,
		-columns => 75,
		-wrap    => 'SOFT',
		-default => '')
	 );

	# Seperation
	$table->MakeBlank();

	# Submit and closure
	$table->MakeSingle($cgi->submit($gal->{'TxtCreate'}));
	$table->MakeBottom();

	# End form
	print $cgi->end_form(), "\n";
}


#####################
# Work Detail
#
# Work detail, will handle the finer points of a works themselves.  This
# includes setting names, genres et cetera.

sub WorkDetail
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $gal      = shift;					# Gallery hash

	my $filter;						# Filter handler
	my $table;						# Table creation
	my $send;						# Mail handle

	my $res;						# Query results
	my $statement;						# Database query statement

	my $gid;						# Gallery id
	my $wid;						# Work id

	my $url;						# Self referencing link
	my $imain;						# Main menu inline
	my $iwork;						# Works listing

	my $notify;						# Notification option

	my $exist;						# Existence
	my $gtype;						# Gallery type
	my $rating;						# Item rating
	my $thumb;						# Thumbnail
	my $votes;						# Votes

	my %system;						# System hash


	#####################
	# Program area

	# Initiate instances
	$table = new eRealms::Table($cgi);


	# Gallery type
	($gtype) = $database->DataGetGalleryType($param->{'GID'}, $param->{'USER'});

	# Primary set
	($wid) = $database->DataGetWorkID($param->{'WORK'}, $param->{'GID'});
	($gid) = $param->{'GID'}; 


	# Update information
	if (defined($param->{'EDIT_WORK'}))
	{
		# Evaluate statement
		eval
		{
			# Common
			# Initiate handle
			$filter = new eRealms::Filter();

			# Existence
			($exist) = $database->DataGetWorkID($param->{'WNAME'}, $param->{'GID'});


			# Main update
			# Description
			$param->{'WDESCR'} = $filter->StripHTML($param->{'WDESCR'});
			$param->{'WDESCR'} = substr(DBI::neat($param->{'WDESCR'}, 252), 1, -1);
			$param->{'WDESCR'} = (length($param->{'WDESCR'}) > 15) ? $param->{'WDESCR'} : undef;

			# Determin value
			$param->{'WNAME'}  = (defined($exist)) ? $param->{'WORK'} : $param->{'WNAME'};


			# Image
			if ($gtype eq 'image')
			{
				# Generate thumbnail
				$thumb = "$gal->{SetUrl}/$gid/$wid/thumb-work-$param->{WTHUMB}.jpg";

				# Update work
				$database->Write("UPDATE Work
					SET WorkName=?,
					    WorkDescription=?,
					    WorkImageThumbnail=?,
					    WorkTimestamp=LOCALTIMESTAMP
					WHERE WorkID=?
					  AND GalleryID=?",

				 $param->{'WNAME'},
				 $param->{'WDESCR'},
				 $thumb,
				 $wid,
				 $gid
				);
			}

			# Webcomics
			elsif ($gtype eq 'comic')
			{
				# Update work
				# Thumbnails of no use, so simply update description
				$database->Write("UPDATE Work
					SET WorkName=?,
					    WorkDescription=?
					WHERE WorkID=?
					  AND GalleryID=?",

				 $param->{'WNAME'},
				 $param->{'WDESCR'},
				 $wid,
				 $gid
				);
			}

			# Journal and Text
			else
			{
				# Preserve text
				my $preserve = (defined($param->{'WPRESERVE'})) ? 'yes' : 'no';

				# Safe HTML
				if ($preserve eq 'no')
				{
					# Initial filter
					$param->{'WTEXT'} = $filter->SafeHTML($param->{'WTEXT'});

					# Create hyperlinks
       		         		$param->{'WTEXT'} =~ s/(http:\/\/.*?)(\s|$)/<A HREF="$1" TARGET="_blank">$gal->{TagHyperlink}<\/A> /gm;
       		         		$param->{'WTEXT'} =~ s/([\w\.\-]*?\@[\w\.\-]*?\..*?)(\s|$)/<A HREF="mailto:$1">$gal->{TagEmail}<\/A> /gm;
				}


				# Additional HTML
				my $update = ($gtype ne 'journal') ? ', WorkTimestamp=LOCALTIMESTAMP': '';
				
				# Update work
				$database->Write("UPDATE Work
					SET WorkName=?,
					    WorkDescription=?,
					    WorkText=?,
					    WorkTextPreserve=?$update
					WHERE WorkID=?
					  AND GalleryID=?",

				 $param->{'WNAME'},
				 $param->{'WDESCR'},
				 $param->{'WTEXT'},
				 $preserve,
				 $wid,
				 $gid
				);


				# Image print formatting
				if ($gtype eq 'text')
				{
					# Associate with hash
					$database->GetHashSystem(\%system);

					# Directory name
					my $dir  = $gal->{'SetRoot'} . '/' . $gid . '/' . $wid;

					# Create instance
					my $page = new eRealms::Home($database, $dir);


					# Create a folder
					mkdir($dir, 0775) unless (-d $dir);


					# Export generation
					# Fetch external details
					my ($gname, $gauthor) = $database->DataGetGalleryBasics($gid);

					# Fetch and export work
					my ($export) = $database->DocumentGetGalleryExport();


					# Simplification
					$param->{'WTEXT'} =~ s/\n/<BR>/gs;
					
					# Search and replace
					$export =~ s/MAUTHOR/$gauthor/gs;
					$export =~ s/MGNAME/$gname/gs;
					$export =~ s/MGID/$gid/gs;
					$export =~ s/MWID/$wid/gs;
					$export =~ s/MBODY/$param->{'WTEXT'}/gs;
					$export =~ s/MGWORK/$param->{'WNAME'}/gs;
					$export =~ s/MTITLE/$param->{'WNAME'}/gs;

					
					# Write temporary file
					$page->FileWrite('export.html', $export);


					# Export to PDF
					# Evaluate clause
					eval
					{
						# Execute command
						system("$system{SetExport} $system{SetExportOpt} $dir/work.pdf $dir/export.html");
					};

					# If block failed
					if ($@)
					{
						# Issue warning
						warn("Error in export: $@");
					}

					# Remove file
					$page->FileUnlink('export.html');
				}
			}

			# General update
			$database->Write("UPDATE Gallery
				SET GalleryUpdated=LOCALTIMESTAMP
				WHERE GalleryID=?
				  AND PuppeteerLogin=?",

			 $param->{'GID'},
			 $param->{'USER'}
			);


			# Mail notification
			if (defined($param->{'NOTIFY'}))
			{
				# Verify for notification
				$database->Pull(\$statement, "SELECT
					p.PuppeteerName  AS \"PuppeteerName\",
					p.PuppeteerEmail AS \"PuppeteerEmail\"
					FROM Puppeteer p, GalleryNotify g
					WHERE g.PuppeteerLogin=p.PuppeteerLogin
					  AND g.GalleryID=?", $gid);


				# Notification applies
				if ($res = $statement->fetchrow_hashref())
				{
					# Fetch additional details
					# Basic gallery details
					my ($gname, $gauthor) = $database->DataGetGalleryBasics($gid);

					# Actual template
					my $message = $database->DocumentGetMailGallery();


					# Continue and repeat as necessary
					do
					{
						# Create instance
						$send = tie(*MAIL, 'eRealms::Mail', $database);

						# Initial settings
						$send->SetSubject("Update to $gname by $gauthor");

						# Recipients
						$send->AddTo("$res->{PuppeteerName} <$res->{PuppeteerEmail}>");

						# Search and replace
						$send->SetSearch(
							MDESCR => $param->{'WDESCR'},
							MANAME => $gauthor,
							MGNAME => $gname,
							MPNAME => $res->{'PuppeteerName'},
							MWNAME => $param->{'WNAME'},
							LWORKA => "$gal->{SetScript}$gal->{LnkGallery}/view/$gid/$wid",
							LGALLA => "$gal->{SetScript}$gal->{LnkGallery}/view/$gid"
						);

						# Send out message
						print MAIL $message;

						# Close and send message
						close(MAIL);
					}
					while ($res = $statement->fetchrow_hashref());
				}

				# End search
				$statement->finish();
			}
			

			# Commit changes
			$database->Commit();
		};


		# If block failed
		if ($@)
		{
			# Issue warning
			warn("Transaction aborted: $@");

			# Undo incomplete changes
			$database->Rollback;
		}
	}

	# Cleanup
	$param->Cleanup($gal->{'TxtDetails'}, 'WPRESERVE', 'NOTIFY');


	# Information gathering
	# Prepare and execute
	$database->Pull(\$statement, "SELECT 
		  WorkName           AS \"WorkName\",
		  WorkDescription    AS \"WorkDescription\",
		  WorkImage          AS \"WorkImage\",
		  WorkImageThumbnail AS \"WorkImageThumbnail\",
		  WorkImageHeight    AS \"WorkImageHeight\",
		  WorkImageWidth     AS \"WorkImageWidth\",
		  WorkText           AS \"WorkText\",
		  WorkTextPreserve   AS \"WorkTextPreserve\",
		  getDate(WorkTimestamp) AS \"WorkTimestamp\"
		FROM Work
		WHERE WorkID=?
		  AND GalleryID=?
		  AND WorkID > 1",

	 $wid,
	 $gid
	);


	# Results successful
	if ($res = $statement->fetchrow_hashref())
	{
		# Calculate rating
		($rating, $votes) = $database->DataGetWorkAvg($gid, $wid);


		# Display information
		# Combined

		# Back track
		$url   = $cgi->url(-relative=>1);
		$imain = $param->EmbedInline(
			USER=>$param->{'USER'}, 
			CRYPT=>$param->{'CRYPT'}
 		 );
		$iwork = $param->EmbedInline(
			USER=>$param->{'USER'}, 
			CRYPT=>$param->{'CRYPT'},
			GID=>$gid,
		 	WORK=>'True'	
		);


		# Title
		$tmpl->ShowMain($gal->{'TagWorkDetail'});

		# Link
		print "<CENTER>",
		      "<A HREF=\"$url$imain\">$gal->{TagGalListing}</A> ",
		      "<A HREF=\"$url$iwork\">$gal->{TagGalWork}</A> ",
		      "</CENTER><BR>";


		# Form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat(), EDIT_WORK=>'True', WORK=>$res->{'WorkName'});


		# Table top
		$table->MakeTop();
		$table->MakeSingle($tmpl->PassTitle($res->{'WorkName'}));
		$table->MakeBlank();


		# Work Name
		$table->MakeValid(
			$tmpl->PassSub($gal->{'TagWrkName'}),
			$cgi->textfield('WNAME', $res->{'WorkName'}, 25, 30)
		 );


		# Image specific
		if (($gtype eq 'image')
		 || ($gtype eq 'comic'))
		{
			# Associate with hash
			$database->GetHashSystem(\%system);


			# Image height and width
			$table->MakeValid(
				$tmpl->PassSub($gal->{'TagWrkWidth'}),
				"<I>$res->{WorkImageWidth}</I>"
			 );	
			$table->MakeValid(
				$tmpl->PassSub($gal->{'TagWrkHeight'}),
				"<I>$res->{'WorkImageHeight'}</I>"
			 );	
			$table->MakeBlank();


			# Thumbnail Selection
			$table->MakeSingle($tmpl->PassTitle($gal->{'TagWrkThumbnail'}));


			# Image galleries only
			if ($gtype eq 'image')
			{
				# Display thumbnails
				my @thumb = qw(neg mid pos);

				# One at a time
				foreach $thumb (@thumb)
				{
					# Selected or not
					if ($res->{'WorkImageThumbnail'} =~ /$thumb/)
					{
						# Default
						$table->MakeValid(
							"<IMG SRC=\"$gal->{SetUrl}/$gid/$wid/thumb-work-$thumb.jpg\" HEIGHT=\"$system{'SetThumbnail'}\" WIDTH=\"$system{'SetThumbnail'}\">",
							"<CENTER><INPUT TYPE=\"radio\" NAME=\"WTHUMB\" VALUE=\"$thumb\" CHECKED></CENTER>",
							"middle"
						 );
					}
					else
					{
						# Non-default
						$table->MakeValid(
							"<IMG SRC=\"$gal->{SetUrl}/$gid/$wid/thumb-work-$thumb.jpg\" HEIGHT=\"$system{'SetThumbnail'}\" WIDTH=\"$system{'SetThumbnail'}\">",
							"<CENTER><INPUT TYPE=\"radio\" NAME=\"WTHUMB\" VALUE=\"$thumb\"></CENTER>",
							"middle"
						 );
					}
				}

				# Spacer
				$table->MakeBlank();
			}
		}

		# Text specific
		else
		{
			# Generate widget
			my $preserve = ($res->{'WorkTextPreserve'} eq 'yes')
			  ? $cgi->checkbox(-name=>'WPRESERVE', -label=>'', -checked=>'on')
			  : $cgi->checkbox(-name=>'WPRESERVE', -label=>'');

			# Preserve text
			$table->MakeValid(
				$tmpl->PassSub($gal->{'TagWrkPreserve'}),
				$preserve);
			$table->MakeBlank();

			# Body of text
			$table->MakeSingle($tmpl->PassTitle($gal->{'TagWrkTxt'}));
			$table->MakeSingle($cgi->textarea(-name=>'WTEXT',-rows=>15,-columns=>75,-wrap=>'SOFT',-default=>$res->{'WorkText'}));

			# Spacer
			$table->MakeBlank();
		}

		# Description
		$table->MakeSingle($tmpl->PassTitle($gal->{'TagWrkDescr'}));		
		$table->MakeSingle($cgi->textarea(-name=>'WDESCR',-rows=>3,-columns=>75,-wrap=>'SOFT',-default=>$res->{'WorkDescription'}));
		$table->MakeBlank();


		# Notification
		# Generate Widget
		$notify = $cgi->checkbox(-name=>'NOTIFY', -label=>'');
		
		# Display
		$table->MakeValid(
			$tmpl->PassSub($gal->{'TagWrkNotify'}),
			$notify);
		$table->MakeBlank();


		# Submission widgets
		$table->MakeValid($cgi->submit($gal->{'TxtUpdate'}), $cgi->reset($gal->{'TxtReset'}));
		$table->MakeBlank();

		# Table bottom
		$table->MakeSingle($tmpl->Pass('TmplRatings',
			MGALLERY => $gid,
			MITEMNUM => $wid,
			MOLVOTES => $votes,
			MRATINGS => $rating,
			MUPDATED => $res->{'WorkTimestamp'},
			MGALLINK => "$gal->{LnkGallery}/view/$gid$imain",
			MWRKLINK => "$gal->{LnkGallery}/view/$gid/$wid$imain")
		 );

		$table->MakeBottom();

		# End form
		print $cgi->end_form, "\n";
	}

	# End query
	$statement->finish();
}


#####################
# Work Summary
#
# Will list all of the works, while allowing the user to add and remove
# them as need be.  This will be similiar to the interface for galleries
# only work specific.

sub WorkSummary
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $gal      = shift;					# Gallery hash
	my $upload   = shift;					# Upload handle

	my $image;						# Image library
	my $option;						# Option handler
	my $page;						# File handler
	my $table;						# Table creation

        my $arrows;						# Arrows for manual
	my $buttons;						# Management buttons
	my $listing;						# List of works
	my $message;						# Notice message

	my $gid;						# Gallery ID
	my $fsort;						# Final sort order
	my $gsort;						# Gallery sorting
	my $gtype;						# Gallery type
	my $wid;						# Work ID
	my $work;						# Individual works
	
	my $url;						# Self referencing link
	my $imain;						# Inline main menu

	my $contents;						# File contents
	my $err;						# Write error message
	my $ext;						# File extension
	my $mime;						# Mime type

	my @works;						# Array of works


	#####################
	# Program area	

	# Gallery type
	($gtype, $gsort) = $database->DataGetGalleryType($param->{'GID'}, $param->{'USER'});


	# Information handling
	# Additions with names greater then 3 
	if ((defined($param->{$gal->{'TxtCreate'}}))
	 && (CheckString($param->{'ADD_WORK'}, 2)))
	{
		# Duplicate check
		($wid)  = $database->DataGetWorkID($param->{'ADD_WORK'}, $param->{'GID'});

		# Safe to proceed
		unless (defined($wid))
		{
			# Evalutate
			eval
			{
				# Database work
				# Create database entry
				$database->Write("INSERT INTO Work
					(GalleryID,
					 WorkName,
					 WorkSort)
					VALUES (?,?,getSort(?))", 

				 $param->{'GID'},
				 $param->{'ADD_WORK'},
				 $param->{'GID'}
				);

				# Update gallery
				$database->Write("UPDATE Gallery
					SET GalleryUpdated=LOCALTIMESTAMP
					WHERE GalleryID=?
					  AND PuppeteerLogin=?",

				 $param->{'GID'},
				 $param->{'USER'}
				);


				# Retreive Work and Gallery ID
				($wid) = $database->DataGet("SELECT CURRVAL('seqWork')");
				($gid) = $param->{'GID'};


				# File work
				# Image galleries only
				if (($gtype eq 'image')
				 || ($gtype eq 'comic'))
				{
					# Create instances
					$page  = new eRealms::Home($database, "$gal->{SetRoot}/$gid/$wid");
					$image = new eRealms::Image($database, "$gal->{SetRoot}/$gid/$wid");


					# Stale entries
					# Remove file
					unlink $page->{'BASEDIR'} if (-f $page->{'BASEDIR'});

					# Remove directory
					$page->DirClear($page->{'BASEDIR'}) if (-d $page->{'BASEDIR'});
			

					# Attempt to create
					# Failed to create
					unless (mkdir($page->{'BASEDIR'}, 0775))
					{
						# Failed
						# Remove database entry
						$database->Rollback();

						# Signal notice
						$message = $gal->{'MsgError'} . $!;
					}

					# Created directory
					else
					{
						# Upload file
						# Valid handle
						if (defined($upload))
						{
							# Determin MIME type
							$mime = $cgi->uploadInfo($param->{'UPLOAD_WORK'})->{'Content-Type'};

							# Determin file name
							$ext = $param->{'UPLOAD_WORK'};
							$ext =~ /^.*\.(\w+)$/;
							$ext = $1;

							# Valid MIME
							if ($mime =~ /^image/i)
							{
								# Retreive image
								while (<$upload>) { $contents = "$contents$_"; }

								# File writing
								# Write image
								$err = $page->FileWrite("work.$ext", $contents);

								# Webcomic consistent link
								if ($gtype eq 'comic')
								{
									# Change base address
									$page->{'BASEDIR'} = "$gal->{SetRoot}/$gid";
									
									# Write copy
									$err = $page->FileWrite("main.$ext", $contents);

									# Reset address
									$page->{'BASEDIR'} = "$gal->{SetRoot}/$gid/$wid";
								}


								# Successful
								if ($err == 1)
								{
									# Create thumbnails
									$image->Thumbnail("work.$ext");

									# Retreive sizes
									my ($width, $height, $size) = $image->Size("work.$ext");


									# Update database
									$database->Write("UPDATE Work
										SET WorkImage=?,
										    WorkImageThumbnail=?,
										    WorkImageHeight=?,
										    WorkImageWidth=?,
										    WorkImageSize=?
										WHERE WorkID=?
										  AND GalleryID=?",

									 "$gal->{SetUrl}/$gid/$wid/work.$ext",
									 "$gal->{SetUrl}/$gid/$wid/thumb-work-mid.$ext",
									 $height,
									 $width,
									 $size,
									 $wid,
									 $gid
									);


									# Message of success
									$message = $gal->{'MsgCreated'};
								}
	
								else
								{
									# Unable to write
									$message = $gal->{'MsgError'} . $err;
								}
	
							}
							else
							{
								# Improper MIME type
								$message = $gal->{'MsgMime'};
							}
						}
					}
				}

				# Nothing to create
				else
				{
					# Message of success
					$message = $gal->{'MsgCreated'};
				}

				# Commit changes
				$database->Commit();
			};

			# If block failed
			if ($@)
			{
				# Issue warning
				warn("Transaction aborted: $@");

				# Undo incomplete changes
				$database->Rollback();
			}
		}
		else
		{
			# Name exists
			$message = $gal->{'MsgExists'};
		}
	}

	# Removal
	if ((defined($param->{$gal->{'TxtDelete'}}))
	 && (defined($param->{'CONFIRM'})))
	{
		# Determine Work/Gallery ID
		($wid)  = $database->DataGetWorkID($param->{'WORK'}, $param->{'GID'});
		($gid)  = $param->{'GID'};

		# Remove
		if (defined($wid))
		{
			# Initialize counter
			my $count = 1;

			# Pull list of works
			$database->GetListWorkSort(\@works, $gid);


			# Evaluate block
			eval
			{
				# Database entry
				$database->Write("DELETE FROM Work
					WHERE WorkID=?
					  AND GalleryID=?", 

				 $wid, 
				 $gid
				);	

				# Loop and re-assign
				foreach $work (@works)
				{
					# Update individual entries to new order
					$database->Write("UPDATE Work
						SET WorkSort=?
						WHERE WorkID=?
						  AND GalleryID=?",
					
					 $count,
					 $work,
					 $gid
				 	);

					# Increment
					$count++;
				}
				

				# Update gallery
				$database->Write("UPDATE Gallery
					SET GalleryUpdated=LOCALTIMESTAMP
					WHERE GalleryID=?
					  AND PuppeteerLogin=?",

				 $gid,
				 $param->{'USER'}
				);


				# Directory structure
				if ($gtype eq 'image')
				{
					# Create instances
					$page  = new eRealms::Home($database, "$gal->{SetRoot}/$gid/$wid");

					# Remove directory
					$page->DirClear($page->{'BASEDIR'}) if (-d $page->{'BASEDIR'});
				}

				# Define message
				$message = $gal->{'MsgRemoved'};

				# Commit changes
				$database->Commit();
			};

			# If block failed
			if ($@)
			{
				# Issue warning
				warn("Transaction aborted: $@");

				# Undo incomplete changes
				$database->Rollback();
			}
		}
	}

	# Movement
	if ((defined($param->{'MUPS'}))
	 || (defined($param->{'MDWN'})))
	{
		# Determine Work/Gallery ID
		($wid)  = $database->DataGetWorkID($param->{'WORK'}, $param->{'GID'});
		($gid)  = $param->{'GID'};

		# Remove
		if (defined($wid))
		{
			# Initialize counter
			my $count  = 1;

			# Current and level
			my $icnt;
			my $lvl;

			# Obstacle
			my $obs;


			# Operation used in query
			my $op     = (defined($param->{'MUPS'})) ? '+' : '-';

			# Affect of movement
			$lvl       = (defined($param->{'MUPS'})) ? $param->{'MUPS'} : $param->{'MDWN'};
			$lvl       = ((defined($lvl)) && ($lvl < 15)) ? $lvl : 1;


			# Current sort number
			my ($wsrt) = $database->DataGetWorkSort($wid, $gid);
			

			# Continue until exhausted
			for ($icnt=$lvl; $icnt > 0; $icnt--)
			{
				# Premature exist
				last if (defined($obs));

				# What is in the way
				($obs)  = $database->DataGet("SELECT WorkID
					FROM Work
					WHERE WorkSort=(? $op $icnt)::INT
					  AND GalleryID=?
					  AND WorkID > 1",
				 $wsrt,
				 $gid
				);
			}

			# Safety	
			$obs = (defined($obs)) ? $obs : 0;


			# Pull list of works
			$database->GetListWorkSort(\@works, $gid);


			# Evaluate block
			eval
			{
				# Loop and re-assign
				foreach $work (@works)
				{
					# Loop changes
					# No change for ends
					last if ($obs == 0);

					# Skip on target
					next if ($work == $wid);


					# Insert before current entry
					if (($work == $obs) && (defined($param->{'MDWN'})))
					{
						# Push in entry ahead
						$database->Write("UPDATE Work
							SET WorkSort=?
							WHERE WorkID=?
							  AND GalleryID=?",
					
						 $count,
						 $wid,
						 $gid
				 		);

						# Increment
						$count++;
					}

					
					# Update individual entries to new order
					$database->Write("UPDATE Work
						SET WorkSort=?
						WHERE WorkID=?
						  AND GalleryID=?",
					
					 $count,
					 $work,
					 $gid
				 	);

					# Increment
					$count++;

					# Insert after current entry
					if (($work == $obs) && (defined($param->{'MUPS'})))
					{
						# Push in entry ahead
						$database->Write("UPDATE Work
							SET WorkSort=?
							WHERE WorkID=?
							  AND GalleryID=?",
					
						 $count,
						 $wid,
						 $gid
				 		);

						# Increment
						$count++;
					}

				}
				
				# Commit changes
				$database->Commit();
			};

			# If block failed
			if ($@)
			{
				# Issue warning
				warn("Transaction aborted: $@");

				# Undo incomplete changes
				$database->Rollback();
			}
		}
	}

	# Always cleanup
	$param->Cleanup('CONFIRM', 'MDWN', 'MUPS');

	# Back track
	$url   = $cgi->url(-relative=>1);
	$imain = $param->EmbedInline(
		USER=>$param->{'USER'}, 
		CRYPT=>$param->{'CRYPT'}
 	 );


	# Create instances
	$table  = new eRealms::Table($cgi);
	$option = new eRealms::Option();


	# Determine sort order
	$fsort = $option->GallerySortReal($gsort);


	# Information display
	# Retrieve list
	$database->GetList(\@works, "SELECT WorkName
		 FROM Work 
		 WHERE GalleryID=?
		   AND WorkID > 1 
		 ORDER BY $fsort", 

	 $param->{'GID'}
	);

	# Widget generation
	$listing = $cgi->scrolling_list('WORK', \@works, $param->{'WORK'}, 10);
	$buttons = $cgi->submit($gal->{'TxtDetails'}) . '<BR><BR>'
		.  $cgi->submit($gal->{'TxtDelete'})
		.  $cgi->checkbox(-name=>'CONFIRM', -label=>'');
        $arrows  = "<INPUT TYPE=\"image\" SRC=\"$gal->{TagArrowDown}\" NAME=\"MDWN\" VALUE=\"1\">"
                .  "&nbsp;"
		.  "<INPUT TYPE=\"image\" SRC=\"$gal->{TagArrowDblDown}\" NAME=\"MDWN\" VALUE=\"5\">"
                .  "&nbsp;&nbsp;"
		.  "<INPUT TYPE=\"image\" SRC=\"$gal->{TagArrowDblUp}\" NAME=\"MUPS\" VALUE=\"5\">"
                .  "&nbsp;"
                .  "<INPUT TYPE=\"image\" SRC=\"$gal->{TagArrowUp}\" NAME=\"MUPS\" VALUE=\"1\">";

	# Display
	# Title
	$tmpl->ShowMain($gal->{'TagWork'});

	# Link
	print "<CENTER><A HREF=\"$url$imain\">$gal->{'TagGalListing'}</A></CENTER><BR>";


	# Form
	if (($gtype eq 'image')
	 || ($gtype eq 'comic'))
	{
		# Multi-part forms for upload capabilities
		print $cgi->start_multipart_form();
		print $param->EmbedNormal($param->Flat()), "\n";
	}

	# Text based galleries
	else
	{
		# Regular forms for reduced abuse
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat()), "\n";
        }


	# Table top
	$table->MakeTop();

	# Notice messages
	if (defined($message))
	{
		# Display through template
		$table->MakeSingle($tmpl->Pass('TmplNotice', MNOTICE=>$message));
		$table->MakeBlank();
	}


	# Gallery listing
	$table->MakeSingle($tmpl->PassSub($gal->{'TagSelectWork'}));
	$table->MakeValid($listing, $buttons, 'middle');

	# Sorting arrows
	if ($gsort eq 'manual')
	{
		# Only display for manual users
       		$table->MakeValid($arrows, '&nbsp;');
	}

	# Additions
	# Image and text concurrence capabale
	if (($gtype eq 'image')
	 || ($gtype eq 'text'))
 	{
		# Seperation
		$table->MakeBlank();

		# Gallery creation
		# Base
		$table->MakeSingle($tmpl->PassTitle($gal->{'TagAddWork'}));
		$table->MakeValid(
			$tmpl->PassSub($gal->{'TagAddWorkName'}),
			$cgi->textfield('ADD_WORK', '', 25, 30)
		 );

		# Upload field
		if ($gtype eq 'image')
		{
			$table->MakeValid(
				$tmpl->PassSub($gal->{'TagAddWorkFile'}),
				$cgi->filefield('UPLOAD_WORK', '', 25, 140)
			 );
		}

		# Submit widget
		$table->MakeSingle($cgi->submit($gal->{'TxtCreate'}));
	}

	# Restrictions based on date
	else
	{
		# Look for works done on the same day
		($wid) = $database->DataGet("SELECT WorkID
			FROM Work
			WHERE GalleryID=?
			  AND WorkID > 1
			  AND WorkTimestamp > DATE_TRUNC('day', LOCALTIMESTAMP)
			  AND WorkTimestamp < DATE_TRUNC('day', LOCALTIMESTAMP + INTERVAL '1 day')",
		 $gid
		 );
		
		# Nothing for that day
		unless (defined($wid))
		{
			# Seperation
			$table->MakeBlank();

			# Gallery creation
			# Base
			$table->MakeSingle($tmpl->PassTitle($gal->{'TagAddWork'}));
			$table->MakeValid(
				$tmpl->PassSub($gal->{'TagAddWorkName'}),
				$cgi->textfield('ADD_WORK', '', 25, 30)
			 );

			# Upload field
			if ($gtype eq 'comic')
			{
				$table->MakeValid(
					$tmpl->PassSub($gal->{'TagAddWorkFile'}),
					$cgi->filefield('UPLOAD_WORK', '', 25, 140)
				 );
			}

			# Submit widget
			$table->MakeSingle($cgi->submit($gal->{'TxtCreate'}));
		}
	}
	
	# Table Bottom
	$table->MakeBottom();

	# End form
	print $cgi->end_form(), "\n";
}
